//+------------------------------------------------------------------+
//|                                               COrdersCounter.mqh |
//|                                                   A.Lopatin 2014 |
//|                                              diver.stv@gmail.com |
//+------------------------------------------------------------------+
#property copyright "A.Lopatin 2014"
#property link      "diver.stv@gmail.com"
#property strict
#property version   "1.00"

//+------------------------------------------------------------------+
//|     Struct for the saving profit data                            |
//+------------------------------------------------------------------+
struct ProfitData
{
public:
   double currency; // Profit in currency of an order
   double points;   // Profit in points 
};

//+------------------------------------------------------------------+
//| Class COrdersCounter for the counting orders by symbol, type     |
//| magic number, mode (trading or history pool)                     |
//+------------------------------------------------------------------+
class COrdersCounter
{
private:
   int orders_magic;                   // Magic Number for the filtering orders. If 0, then doesn't filter.
   string orders_symbol;               // Currency symbol for the filtering orders. If "", then doesn't filter.
   int orders_mode;                    // Type of the pool: MODE_TRADES(default) - trading, MODE_HISTORY - history pool
   int buys;                           // Count of the buy positions
   int sells;                          // Count of the sell poistions
   int buy_limits;                     // Count of the buy limit orders
   int sell_limits;                    // Count of the sell limit orders
   int buy_stops;                      // Count of the buy stop orders
   int sell_stops;                     // Count of the sell stop orders
   ProfitData total_profit;            // Profit of the counted orders
   double total_volume;                // Total volume - number of lots for all counted orders
   double market_volume;               // Total volume - number of lots for counted market positions
   double getAsk(string symbol);
   double getBid(string symbol);
   double getPoint(string symbol);
public:
   COrdersCounter(void);
   COrdersCounter(int magic, string symbol, int counter_mode = MODE_TRADES);
   void refreshData(void);
   int getOrdersCount(int type);
   int getLastOrder();
   int getFirstOrder();
   ProfitData getTotalProfit();
   double getMarketVolume();
   double getTotalVolume();
   void setMagicNumber(int magic);
   void setSymbol(string symbol);
   void setMode(int mode);
   int getMagicNumber();
   string getSymbol();
   int getMode();
};

//+------------------------------------------------------------------+
//| Default constructor COrdersCounter                               |
//+------------------------------------------------------------------+
COrdersCounter::COrdersCounter(void)
{
   this.orders_magic = 0;
   this.orders_symbol = Symbol();
   this.orders_mode = MODE_TRADES;
   this.buys = 0;
   this.sells = 0;
   this.buy_limits = 0;
   this.sell_limits = 0;
   this.buy_stops = 0;
   this.sell_stops = 0;
   this.total_profit.currency = 0.0;
   this.total_profit.points = 0.0;
   this.total_volume = 0.0;
   this.market_volume = 0.0;
}

//+------------------------------------------------------------------+
//|  Constructor COrdersCounter                                      |
//|  Arguments: magic - magic number of orders,                      |
//|               if magic = 0, then it counts for any magics        |
//|            symbol - symbol of currency for orders                |
//|               if symbol = "", then it counts for all currencies  |
//|            counter mode - type of orders list:                   |
//|               MODE_TRADES(default)- order selected from trading  |
//|               pool(opened and pending orders)                    |
//|               MODE_HISTORY - order selected from history pool    |
//|               (closed and canceled order).                       |
//+------------------------------------------------------------------+
COrdersCounter::COrdersCounter(int magic, string symbol, int counter_mode = MODE_TRADES)
{
   this.orders_magic = magic;
   this.orders_symbol = symbol;
   this.orders_mode = counter_mode;
   this.buys = 0;
   this.sells = 0;
   this.buy_limits = 0;
   this.sell_limits = 0;
   this.buy_stops = 0;
   this.sell_stops = 0;
   this.total_profit.currency = 0.0;
   this.total_profit.points = 0.0;
   this.total_volume = 0.0;
   this.market_volume = 0.0;
}

//+------------------------------------------------------------------+
//|  The method renews data for orders.                              |
//|  It is should called before a calling get...() method.           |
//+------------------------------------------------------------------+
void COrdersCounter::refreshData(void)
{
    int orders_total = OrdersTotal();
    double point = MarketInfo(this.orders_symbol, MODE_POINT), price = 0.0;
    string order_symbol = "";
    
    if( this.orders_mode == MODE_HISTORY )
      orders_total = OrdersHistoryTotal();
      
    this.total_profit.currency = 0.0;
    this.total_profit.points = 0;
    this.total_volume = 0.0;
    this.market_volume = 0.0;
    
    for( int i = 0; i < orders_total; i++ )
    {
        if( OrderSelect( i, SELECT_BY_POS, this.orders_mode ) )
        {
            order_symbol = OrderSymbol();
            
            if( order_symbol != this.orders_symbol && this.orders_symbol != "" )
                continue;
                
            if( OrderMagicNumber() == this.orders_magic || this.orders_magic == 0 )
            {
               point = getPoint(order_symbol);
               switch(OrderType())
               {
                  case OP_BUY:
                     this.buys++;
                     this.market_volume += OrderLots();
                     this.total_profit.currency += (OrderProfit() + OrderSwap() + OrderCommission());
                     price = getBid(order_symbol);
                     if( point > 0.0 && price > 0.0 )
                        this.total_profit.points += (price - OrderOpenPrice())/point;
                     break;
                  case OP_SELL:
                     this.sells++;
                     this.market_volume += OrderLots();
                     this.total_profit.currency += (OrderProfit() + OrderSwap() + OrderCommission());
                     price = getAsk(order_symbol);
                     if( point > 0.0 && price > 0.0 )
                        this.total_profit.points += (OrderOpenPrice() - price)/point;
                     break;
                  case OP_BUYLIMIT:
                     this.buy_limits++;
                     break;
                  case OP_SELLLIMIT:
                     this.sell_limits++;
                     break;
                  case OP_BUYSTOP:
                     this.buy_stops++;
                     break;
                  case OP_SELLSTOP:
                     this.sell_stops++;
                     break;
                  default:
                     break;
               }
               
               total_volume += OrderLots();
            }
        }
    }
}

//+------------------------------------------------------------------+
//|  The method retrieves the count(int) of orders                   |
//|  Arguments: type - type of the counting orders. Allowed values:  |
//|               OP_BUY = 0 - market buy order                      |
//|               OP_SELL = 1 - market sell orders                   |
//|               OP_BUYLIMIT = 2 - pending buy limits orders        |
//|               OP_SELLLIMIT = 3 - pending sell limits orders      |
//|               OP_BUYSTOP = 4 - pending buy stop orders           |
//|               OP_SELLSTOP = 5 - pending sell stop orders         |
//+------------------------------------------------------------------+
int COrdersCounter::getOrdersCount(int type)
{
   int count = 0;
   switch(type)
   {
      case -1:
         count = this.buys + this.sells + this.buy_limits + this.sell_limits + this.buy_stops + this.sell_stops;
         break;
      case OP_BUY:
         count = this.buys;
         break;
      case OP_SELL:
         count = this.sells;
         break;
      case OP_BUYLIMIT:
         count = this.buy_limits;
         break;
      case OP_SELLLIMIT:
         count = this.sell_limits;
         break;
      case OP_BUYSTOP:
         count = this.buy_stops;
         break;
      case OP_SELLSTOP:
         count = this.sell_stops;
         break;
      default:
         break;
    }
    return(count);
}

//+------------------------------------------------------------------+
//| The method retrieves getLastOrder() the ticket of the last order.| 
//| It sorts the orders either by the open time for trading poll,    |
//| either by the close time for  history pool.                      |
//+------------------------------------------------------------------+
int COrdersCounter::getLastOrder()
{
    int     orders_total  = 0, ticket  = 0;
    datetime last_time = 0, order_time = 0;
    
    orders_total = OrdersTotal();
    if( this.orders_mode == MODE_HISTORY )
      orders_total = OrdersHistoryTotal();
      
    for( int i = 0; i < orders_total; i++ )
    {
        if( OrderSelect( i, SELECT_BY_POS, this.orders_mode ) )
        {
            if( OrderSymbol() != this.orders_symbol && this.orders_symbol != "" )
                continue;
                
            if( OrderMagicNumber() == this.orders_magic || this.orders_magic == 0 )
            {
                order_time = this.orders_mode == MODE_HISTORY ? OrderCloseTime(): OrderOpenTime();
                if( order_time > last_time )
                {
                    last_time = this.orders_mode == MODE_HISTORY ? OrderCloseTime(): OrderOpenTime();
                    ticket = OrderTicket();
                }
            }
        }
    }//end for
    
    return(ticket);
}

//+------------------------------------------------------------------+
//|The method retrieves getLastOrder() the ticket of the first order.| 
//| It sorts the orders either by the open time for trading poll,    |
//| either by the close time for  history pool.                      |
//+------------------------------------------------------------------+
int COrdersCounter::getFirstOrder()
{
    int     orders_total  = 0, ticket  = 0;
    datetime last_time     = 2*TimeCurrent(), order_time = 0;
    
    orders_total = OrdersTotal();
    if( this.orders_mode == MODE_HISTORY )
      orders_total = OrdersHistoryTotal();
      
    for( int i = 0; i < orders_total; i++ )
    {
        if( OrderSelect( i, SELECT_BY_POS, this.orders_mode ) )
        {
            if( OrderSymbol() != this.orders_symbol && this.orders_symbol != "" )
                continue;
                
            if( OrderMagicNumber() == this.orders_magic || this.orders_magic == 0 )
            {
                order_time = this.orders_mode == MODE_HISTORY ? OrderCloseTime(): OrderOpenTime();
                if( order_time < last_time )
                {
                    last_time = this.orders_mode == MODE_HISTORY ? OrderCloseTime(): OrderOpenTime();
                    ticket = OrderTicket();
                }
            }
        }
    }//end for
    
    return(ticket);
}

//+------------------------------------------------------------------+
//| The private method getPoint() retrives point value for symbol.   |
//|  Arguments: symbol - symbol of the currency                      |
//+------------------------------------------------------------------+
double COrdersCounter::getPoint(string symbol) 
{
   double point = 0.0;
   
   point = MarketInfo(symbol, MODE_POINT);
   double digits = NormalizeDouble( MarketInfo(symbol, MODE_DIGITS),0 );
   
   if( digits == 3 || digits == 5 )
      return(point*10.0);
   
   return(point);
}

//+------------------------------------------------------------------+
//|The private method getAsk() retrieves Ask price value for symbol. |
//|  Arguments: symbol - symbol of the currency                      |
//+------------------------------------------------------------------+
double COrdersCounter::getAsk(string symbol) 
{
   RefreshRates();
   double price = MarketInfo(symbol, MODE_ASK);
   return(price);
}

//+------------------------------------------------------------------+
//| The private method getBid() retrieves Bid price value for symbol.|
//|  Arguments: symbol - symbol of the currency                      |
//+------------------------------------------------------------------+
double COrdersCounter::getBid(string symbol) 
{
   RefreshRates();
   double price = MarketInfo(symbol, MODE_BID);
   return(price);
}

//+-----------------------------------------------------------------------------+
//| The method getTotalProfit() retrieves data of the profit for counted orders |
//| Return value:struct ProfitData, containing the profit values in currency    | 
//| and points.                                                                 |
//+-----------------------------------------------------------------------------+
ProfitData COrdersCounter::getTotalProfit()
{
   return(this.total_profit);
}

//+-----------------------------------------------------------------------------------------+
//| The method getMarketVolume() retrieves data of the total volume(lots) for market orders |
//+-----------------------------------------------------------------------------------------+
double COrdersCounter::getMarketVolume()
{
   return(this.market_volume);
}

//+-----------------------------------------------------------------------------------------+
//| The method getTotalVolume() retrieves data of the total volume(lots) for all orders     |
//+-----------------------------------------------------------------------------------------+
double COrdersCounter::getTotalVolume()
{
   return(this.total_volume);
}

//+------------------------------------------------------------------+
//| The method setMagicNumber() sets a value for the Magic Number    |
//| for counting orders                                              |
//|   Arguments: magic - integer value for the MagicNumber           |
//+------------------------------------------------------------------+
void COrdersCounter::setMagicNumber(int magic)
{
   this.orders_magic = magic;
}

//+-------------------------------------------------------------+
//| The method setSymbol() sets a symbol for counting orders    |
//|  Arguments: symbol - symbol of the currency                 |
//+-------------------------------------------------------------+
void COrdersCounter::setSymbol(string symbol)
{
   this.orders_symbol = symbol;
}

//+------------------------------------------------------------------------+
//| The method setMode() sets pool: trading, historyfor counting orders    |
//| Arguments: mode - type of the pool:                                    |
//|             MODE_TRADES(default)- order selected from trading pool     |
//|              (opened and pending orders)                               |
//|             MODE_HISTORY - order selected from history pool            |
//|              (closed and canceled order).                              |
//+------------------------------------------------------------------------+  
void COrdersCounter::setMode(int mode)
{
   this.orders_mode = mode;
}

//+----------------------------------------------------------------------+
//| The method getMagicNumber() retieves a value for the Magic Number    |
//+------------------------------------------------------------------+
int COrdersCounter::getMagicNumber()
{
   return(this.orders_magic);
}

//+------------------------------------------------------------------+
//| The method getSymbol() retrieves a symbol for counting orders    |
//+------------------------------------------------------------------+
string COrdersCounter::getSymbol()
{
   return(this.orders_symbol);
}

//+------------------------------------------------------------------------+
//| The method getMode() retives pool type : trading, history              |
//+------------------------------------------------------------------------+  
int COrdersCounter::getMode()
{
   return(this.orders_mode);
}